@@ -15,7 +15,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import (Callable, Optional, NamedTuple, Union, TYPE_CHECKING)
+from typing import (Callable, Dict, Optional, NamedTuple, Tuple, Union, TYPE_CHECKING)
 
 from .ap import (APAddressBase, AccessPort)
 from .cortex_m import CortexM
@@ -42,7 +42,17 @@ SYSTEM_CLASS = 0xf # CoreLink, PrimeCell, or other system component with no stan
 #  [11:8] continuation
 #  [6:0]  ID
 ARM_ID = 0x43b
+ARM_CHINA_ID = 0xa75
 FSL_ID = 0x00e
+STM_ID = 0x020
+
+## Map of JEP106 IDs to vendor name.
+VENDOR_NAMES_MAP: Dict[int, str] = {
+    ARM_ID: "Arm",
+    ARM_CHINA_ID: "Arm China",
+    FSL_ID: "NXP",
+    STM_ID: "ST",
+}
 
 # CoreSight devtype
 #  Major Type [3:0]
@@ -81,7 +91,7 @@ class CmpInfo(NamedTuple):
     factory: Optional[Union[ComponentFactory, APFactory]]
 
 ## Map from (designer, class, part, devtype, archid) to component name, product name, and factory.
-COMPONENT_MAP = {
+COMPONENT_MAP: Dict[Tuple[int, int, Optional[int], Optional[int], int], CmpInfo] = {
   # Archid-only entries
   # Designer|Component Class |Part  |Type |Archid           |Name              |Product    |Factory
     (ARM_ID, CORESIGHT_CLASS, None,  None, 0x0a00) : CmpInfo('RASv1',           None,       None                ),
@@ -184,13 +194,24 @@ COMPONENT_MAP = {
     (ARM_ID, CORESIGHT_CLASS, 0xd21, 0x00, 0x2a04) : CmpInfo('SCS',             'M33',      CortexM_v8M.factory ),
     (ARM_ID, CORESIGHT_CLASS, 0xd21, 0x13, 0x4a13) : CmpInfo('ETM',             'M33',      None                ),
     (ARM_ID, CORESIGHT_CLASS, 0xd21, 0x11, 0)      : CmpInfo('TPIU',            'M33',      TPIU.factory        ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x16, 0x0a06) : CmpInfo('PMU',             'M55',      None                ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x00, 0x0a07) : CmpInfo('EWIC',            'M55',      None                ),
     (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x43, 0x1a01) : CmpInfo('ITM',             'M55',      ITM.factory         ),
     (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x00, 0x1a02) : CmpInfo('DWT',             'M55',      DWTv2.factory       ),
     (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x00, 0x1a03) : CmpInfo('BPU',             'M55',      FPB.factory         ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x14, 0x1a14) : CmpInfo('CTI',             'M55',      None                ),
     (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x00, 0x2a04) : CmpInfo('SCS',             'M55',      CortexM_v8M.factory ),
     (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x11, 0)      : CmpInfo('TPIU',            'M55',      TPIU.factory        ),
     (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x13, 0x4a13) : CmpInfo('ETM',             'M55',      None                ),
-    (ARM_ID, CORESIGHT_CLASS, 0xd22, 0x16, 0x0a06) : CmpInfo('PMU',             'M55',      None                ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x16, 0x0a06) : CmpInfo('PMU',             'M85',      None                ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x00, 0x0a07) : CmpInfo('EWIC',            'M85',      None                ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x43, 0x1a01) : CmpInfo('ITM',             'M85',      ITM.factory         ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x00, 0x1a02) : CmpInfo('DWT',             'M85',      DWTv2.factory       ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x00, 0x1a03) : CmpInfo('BPU',             'M85',      FPB.factory         ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x14, 0x1a14) : CmpInfo('CTI',             'M85',      None                ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x00, 0x2a04) : CmpInfo('SCS',             'M85',      CortexM_v8M.factory ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x11, 0)      : CmpInfo('TPIU',            'M85',      TPIU.factory        ),
+    (ARM_ID, CORESIGHT_CLASS, 0xd23, 0x13, 0x4a13) : CmpInfo('ETM',             'M85',      None                ),
     (ARM_ID, CORESIGHT_CLASS, 0xd31, 0x31, 0x0a31) : CmpInfo('MTB',             'M35P',     None                ),
     (ARM_ID, CORESIGHT_CLASS, 0xd31, 0x43, 0x1a01) : CmpInfo('ITM',             'M35P',     ITM.factory         ),
     (ARM_ID, CORESIGHT_CLASS, 0xd31, 0x00, 0x1a02) : CmpInfo('DWT',             'M35P',     DWTv2.factory       ),
@@ -214,6 +235,17 @@ COMPONENT_MAP = {
     (ARM_ID, GENERIC_CLASS,   0x00d, 0x00, 0)      : CmpInfo('SCS',             'SC000',    CortexM.factory     ),
     (ARM_ID, GENERIC_CLASS,   0x00e, 0x00, 0)      : CmpInfo('FPB',             'v7-M',     FPB.factory         ),
     (ARM_ID, SYSTEM_CLASS,    0x101, 0x00, 0)      : CmpInfo('TSGEN',           None,       None                ), # Timestamp Generator
+    (ARM_ID, SYSTEM_CLASS,    0x580, 0x00, 0)      : CmpInfo('U55',             None,       None                ), # Ethos U55 NPU
     (FSL_ID, CORESIGHT_CLASS, 0x000, 0x04, 0)      : CmpInfo('MTBDWT',          None,       None                ),
+    (STM_ID, SYSTEM_CLASS,    0x000, 0x00, 0)      : CmpInfo('DBGMCU',          None,       None                ),
+  # Designer      |Component Class |Part  |Type |Archid           |Name        |Product    |Factory
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x31, 0x0a31) : CmpInfo('MTB',       'Star-MC1', None                ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x43, 0x1a01) : CmpInfo('ITM',       'Star-MC1', ITM.factory         ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x00, 0x1a02) : CmpInfo('DWT',       'Star-MC1', DWTv2.factory       ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x00, 0x1a03) : CmpInfo('BPU',       'Star-MC1', FPB.factory         ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x14, 0x1a14) : CmpInfo('CTI',       'Star-MC1', None                ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x00, 0x2a04) : CmpInfo('SCS',       'Star-MC1', CortexM_v8M.factory ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x13, 0x4a13) : CmpInfo('ETM',       'Star-MC1', None                ),
+    (ARM_CHINA_ID, CORESIGHT_CLASS, 0x132, 0x11, 0)      : CmpInfo('TPIU',      'Star-MC1', TPIU.factory        ),
     }
 
