@@ -1,6 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2020 Arm Limited
-# Copyright (c) 2021 Chris Reed
+# Copyright (c) 2021-2022 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,24 +18,69 @@
 from time import sleep
 import logging
 from typing import (Callable, Collection, Dict, List, Optional, overload, Sequence, Set, TYPE_CHECKING, Tuple, Union)
-from typing_extensions import Literal
-
-from pyocd.probe.pydapaccess.dap_access_api import DAPAccessIntf
+from typing_extensions import (Literal, Protocol)
 
 from .debug_probe import DebugProbe
 from ..core import exceptions
 from ..core.plugin import Plugin
+from ..core.options import OptionInfo
 from .pydapaccess import DAPAccess
 from ..board.mbed_board import MbedBoard
-from ..board.board_ids import BOARD_ID_TO_INFO
+from ..board.board_ids import (BoardInfo, BOARD_ID_TO_INFO)
 
 if TYPE_CHECKING:
+    from types import TracebackType
     from ..board.board import Board
 
 LOG = logging.getLogger(__name__)
 TRACE = LOG.getChild("trace")
 TRACE.setLevel(logging.CRITICAL)
 
+class _OpenableProtocol(Protocol):
+    @property
+    def is_open(self) -> bool:
+        ...
+
+    def open(self) -> None:
+        ...
+
+    def close(self) -> None:
+        ...
+
+class _TemporaryOpen:
+    """@brief Context manager to ensure the device is open for a short time."""
+
+    def __init__(self, device: _OpenableProtocol, suppress_exceptions: bool = True) -> None:
+        self._device = device
+        self._suppress_exceptions = suppress_exceptions
+        self._did_open_link: bool = False
+
+    def __enter__(self) -> "_TemporaryOpen":
+        try:
+            # Temporarily open the device if not already opened.
+            if not self._device.is_open:
+                self._device.open()
+                self._did_open_link = True
+        except (DAPAccess.Error, exceptions.Error) as err:
+            if not self._suppress_exceptions:
+                raise
+            else:
+                LOG.debug("suppressing error from attempting to open device %s: %s", self._device, err)
+
+        return self
+
+    def __exit__(self, exc_type: Optional[type], exc_value: Optional[Exception], traceback: Optional["TracebackType"]) -> bool:
+        # Close the device if we had to open it.
+        if self._did_open_link:
+            self._device.close()
+
+        # Check for and possibly suppress an exception.
+        if (exc_type is not None) and issubclass(exc_type, exceptions.Error):
+            if self._suppress_exceptions:
+                return True
+
+        return False
+
 class CMSISDAPProbe(DebugProbe):
     """@brief Wraps a pydapaccess link as a DebugProbe.
 
@@ -85,7 +130,11 @@ class CMSISDAPProbe(DebugProbe):
     DAPLINK_VIDPID = (0x0d28, 0x0204)
 
     @classmethod
-    def get_all_connected_probes(cls, unique_id: str = None, is_explicit: bool = False) -> Sequence["DebugProbe"]:
+    def get_all_connected_probes(
+                cls,
+                unique_id: Optional[str] = None,
+                is_explicit: bool = False
+            ) -> Sequence["DebugProbe"]:
         try:
             return [cls(dev) for dev in DAPAccess.get_connected_devices()]
         except DAPAccess.Error as exc:
@@ -102,8 +151,8 @@ class CMSISDAPProbe(DebugProbe):
         except DAPAccess.Error as exc:
             raise cls._convert_exception(exc) from exc
 
-    def __init__(self, device: DAPAccessIntf) -> None:
-        super(CMSISDAPProbe, self).__init__()
+    def __init__(self, device: DAPAccess) -> None:
+        super().__init__()
         self._link = device
         self._supported_protocols: List[DebugProbe.Protocol] = []
         self._protocol: Optional[DebugProbe.Protocol] = None
@@ -127,13 +176,7 @@ class CMSISDAPProbe(DebugProbe):
 
     @property
     def description(self) -> str:
-        try:
-            # self.board_id may be None.
-            board_info = BOARD_ID_TO_INFO[self.board_id]
-        except KeyError:
-            return self.vendor_name + " " + self.product_name
-        else:
-            return "{0} [{1}]".format(board_info.name, board_info.target)
+        return self.vendor_name + " " + self.product_name
 
     @property
     def vendor_name(self) -> str:
@@ -164,18 +207,52 @@ class CMSISDAPProbe(DebugProbe):
     def capabilities(self) -> Set[DebugProbe.Capability]:
         return self._caps
 
+    @property
+    def associated_board_info(self) -> Optional["BoardInfo"]:
+        """@brief Info about the board associated with this probe, if known."""
+        # Get internal board info if available.
+        if (self.board_id is not None) and (self.board_id in BOARD_ID_TO_INFO):
+            info = BOARD_ID_TO_INFO[self.board_id]
+        else:
+            info = None
+
+        with _TemporaryOpen(self._link):
+            if self._link.supports_board_and_target_names:
+                # Get v2.1 board and target info values.
+                vendor, board = self._link.board_names
+                _, part_number = self._link.target_names
+
+                # Use the target from internal board info in preference, so built-in targets take
+                # precedence over DFPs (because the probe will only report part numbers from DFPs).
+                if info and info.target:
+                    target_device_name = info.target
+                elif part_number:
+                    target_device_name = part_number.lower().replace("-", "_")
+                else:
+                    target_device_name = None
+
+                # If we have either target type or board then construct the board info.
+                if target_device_name or board:
+                    # Vendor can be None, but the BoardInfo must have a valid name.
+                    if not board:
+                        assert target_device_name
+                        board = "Generic " + (part_number or target_device_name)
+
+                    info = BoardInfo(name=board, target=target_device_name, vendor=vendor)
+
+        return info
+
     def create_associated_board(self) -> Optional["Board"]:
         assert self.session is not None
 
-        # Only support associated Mbed boards for DAPLink firmware. We can't assume other
-        # CMSIS-DAP firmware is using the same serial number format, so we cannot reliably
-        # extract the board ID.
-        if self.board_id is not None:
-            return MbedBoard(self.session, board_id=self.board_id)
-        else:
-            return None
+        board_info = self.associated_board_info
+        if self.board_id or board_info:
+            return MbedBoard(self.session, board_info=board_info, board_id=self.board_id)
+        return None
 
     def open(self) -> None:
+        if self._is_open:
+            return
         assert self.session
         try:
             TRACE.debug("trace: open")
@@ -184,13 +261,28 @@ class CMSISDAPProbe(DebugProbe):
             self._is_open = True
             self._link.set_deferred_transfer(self.session.options.get('cmsis_dap.deferred_transfers'))
 
+            if self._link.supports_board_and_target_names:
+                board_names = self._link.board_names
+                target_names = self._link.target_names
+                if board_names != (None, None):
+                    LOG.debug("Board: %s %s", board_names[0] or "(no vendor)", board_names[1] or "(no name)")
+                if target_names != (None, None):
+                    LOG.debug("Target: %s %s", target_names[0] or "(no vendor)", target_names[1] or "(no name)")
+
             # Read CMSIS-DAP capabilities
-            self._capabilities = self._link.identify(DAPAccess.ID.CAPABILITIES)
+            caps_value = self._link.identify(DAPAccess.ID.CAPABILITIES)
+            if not isinstance(caps_value, int):
+                raise exceptions.ProbeError(f"probe {self.unique_id} returned invalid capabilities")
+            self._capabilities = caps_value
             self._supported_protocols = [DebugProbe.Protocol.DEFAULT]
             if self._capabilities & self.SWD_CAPABILITY_MASK:
                 self._supported_protocols.append(DebugProbe.Protocol.SWD)
             if self._capabilities & self.JTAG_CAPABILITY_MASK:
                 self._supported_protocols.append(DebugProbe.Protocol.JTAG)
+            # Warn if neither SWD nor JTAG is supported.
+            if (self._capabilities & (self.SWD_CAPABILITY_MASK | self.JTAG_CAPABILITY_MASK)) == 0:
+                LOG.warning("probe %s reported capabilities indicating it supports neither SWD nor JTAG",
+                        self.unique_id)
 
             self._caps = {
                 self.Capability.SWJ_SEQUENCE,
@@ -206,6 +298,8 @@ class CMSISDAPProbe(DebugProbe):
             raise self._convert_exception(exc) from exc
 
     def close(self) -> None:
+        if not self._is_open:
+            return
         try:
             TRACE.debug("trace: close")
 
@@ -235,8 +329,9 @@ class CMSISDAPProbe(DebugProbe):
             raise self._convert_exception(exc) from exc
 
         # Read the current mode and save it.
-        actualMode = self._link.get_swj_mode()
-        self._protocol = self._PORT_TO_PROTOCOL[actualMode]
+        actual_mode = self._link.get_swj_mode()
+        assert actual_mode is not None
+        self._protocol = self._PORT_TO_PROTOCOL[actual_mode]
 
     def swj_sequence(self, length: int, bits: int) -> None:
         TRACE.debug("trace: swj_sequence(length=%i, bits=%x)", length, bits)
@@ -483,7 +578,7 @@ class CMSISDAPProbe(DebugProbe):
         try:
             self._link.reg_write_repeat(len(values), ap_reg, values, dap_index=0)
             TRACE.debug("trace: write_ap_multi(addr=%#010x, (%i)[%s])", addr, len(values),
-                    ", ".join(["%#010x" % v for v in values]))
+                   ", ".join(["%#010x" % v for v in values]))
         except DAPAccess.Error as exc:
             TRACE.debug("trace: write_ap_multi(addr=%#010x, (%i)[%s]) -> error(%s)", addr, len(values),
                     ", ".join(["%#010x" % v for v in values]), exc)
@@ -546,3 +641,13 @@ class CMSISDAPProbePlugin(Plugin):
     @property
     def description(self):
         return "CMSIS-DAP debug probe"
+
+    @property
+    def options(self):
+        """@brief Returns CMSIS-DAP probe options."""
+        return [
+            OptionInfo('cmsis_dap.deferred_transfers', bool, True,
+                "Whether the CMSIS-DAP probe backend will use deferred transfers for improved performance."),
+            OptionInfo('cmsis_dap.limit_packets', bool, False,
+                "Restrict CMSIS-DAP backend to using a single in-flight command at a time."),
+            ]
